;##############################################################################
;
; Copyright (C) 2013, Michele Cappellari
; E-mail: cappellari_at_astro.ox.ac.uk
;
; Updated versions of the software are available from my web page
; http://purl.org/cappellari/software
;
; This software is provided as is without any warranty whatsoever.
; Permission to use, for non-commercial purposes is granted.
; Permission to modify for personal or internal use is granted,
; provided this copyright and disclaimer are included unchanged
; at the beginning of the file. All other rights are reserved.
;
;##############################################################################
;+
; NAME:
;   CAP_POLYFIT_1D
;
; PURPOSE:
;   Fit a univariate polynomial of given DEGREE to a set of points
;   (X,Y), assuming errors SIGY in the Y variable only.
;   It differs from IDL's POLY_FIT routine as one can input WEIGHTS 
;   rather than SIGY and the routine allows for zero weights.
;
;   For example with DEGREE=1 this program fits a line
;
;       y = a + b*x
;
;   while with DEGREE=2 the program fits a quadratic function
;
;       y = a + b*x + c*x^2
;
; CALLING SEQUENCE:
;   yfit = CAP_POLYFIT_1D(x, y, degree, SIGY=sigy, COEFF=coeff)
;
; MODIFICATION HISTORY:
;   V1.0: Michele Cappellari, Oxford, 30 October 2013
;-
;##############################################################################
function cap_polyfit_1d, x, y, degree, $
    SIGY=sigy, WEIGHTS=weights, COEFF=coeff
COMPILE_OPT IDL2    
ON_ERROR, 2

    sw = ~n_elements(weights) ? $ 
        (~n_elements(sigy) ? 1d : 1d/sigy) : sqrt(weights)
    
    a = (c = dblarr(degree+1,n_elements(x)))
    for j=0,degree do begin
        c[j,*] = x^j
        a[j,*] = c[j,*]*sw
    endfor
    coeff = LA_LEAST_SQUARES(a,y*sw)
    
    return, reform(c ## coeff)
end
;----------------------------------------------------------------------------------
